import os                                   # Standard Library: Miscellaneous Operating System Interfaces
import sys                                  # Standard Library: System Specific Parameters and Functions

pkg_dir = os.path.abspath(os.path.join(os.path.dirname( __file__ ), '..'))
sys.path.append(pkg_dir)                # Add SNAP-Functions to the library 

# SNAP-Function sen2cor: L1C to L2A
from snappy_sen2cor import sen2corOp
    
# Basic Functions used in multiple scripts
from snappy_basic import basicOp 

def runScript(s2_path, para=10, trg_path='DEFAULT'):
    
    # Find all S2 datasets
    s2_L1C_paths = []
    dir_name_list = basicOp().searchForFolders(s2_path)
    for dir_name in dir_name_list:
        s2_path_tmp = os.path.join(s2_path,dir_name)
        if basicOp().s2folder(s2_path_tmp) == True:
            s2_L1C_paths.append(s2_path_tmp)
            print("INFO : S2-Directory found in {}".format(s2_path_tmp) )
    
    # If no S2 datasets has been found: Error & Exit        
    if len(s2_L1C_paths)==0:
        print("ERROR: No valid Sentinel 2 Dataset found.")
        sys.exit(1)
    
    # Run Operation for all S2 datasets
    for src_path in s2_L1C_paths:
        sen2corOp().sen2cor(src_path,para,trg_path)
        
if __name__ == "__main__":
    
    if len(sys.argv) == 4:
        print("INFO : Input Directory {}".format(sys.argv[1]))
        print("INFO : Resolution {}m".format(sys.argv[2]))
        print("INFO : Output Directory {}".format(sys.argv[3]))
        runScript(sys.argv[1],sys.argv[2],sys.argv[3])
    elif len(sys.argv) == 3:
        print("INFO : Input Directory {}".format(sys.argv[1]))
        print("INFO : Resolution {}m".format(sys.argv[2]))    
        print("INFO : Output Directory {}".format(sys.argv[1]))
        runScript(sys.argv[1],sys.argv[2])
    elif len(sys.argv) == 2:
        print("INFO : Input Directory {}".format(sys.argv[1]))
        print("INFO : Resolution 10m")    
        print("INFO : Output Directory {}".format(sys.argv[1]))        
        runScript(sys.argv[1])   
    elif len(sys.argv) > 5:
        print("ERROR: Too many input arguments!")
        sys.exit(1)        
    else:
        print("ERROR: Not enough input arguments!")
        sys.exit(1)