### Enter in Command Prompt Window
### -------------------------------------------------------------------------------------------
### For Windows:
###                python C:/S2-Python/SNAP/examples/run_sen2cor.py
### -------------------------------------------------------------------------------------------
### For Linux: 
###                source /home/pi/sen2cor/L2A_Bashrc
###                python /home/pi/S2-Python/SNAP/examples/run_sen2cor.py

'''
-----------------------------------------------------------------------------------------------
Import of different packages
-----------------------------------------------------------------------------------------------
'''
# Import required libraries from python
import os
import sys
import platform

import xml.etree.ElementTree as ET      # XML Parser

'''
-----------------------------------------------------------------------------------------------
Class: sen2corOp
-----------------------------------------------------------------------------------------------
'''

class sen2corOp():
    
        
    # Update configuration file of sen2cor
    @staticmethod
    def updateConfigFile(xml_conf_file,trg_dir):
        xml_tree = ET.parse(xml_conf_file)
        xml_root = xml_tree.getroot()
        for xml_trg_dir in xml_root.iter('Target_Directory'):
            xml_trg_dir.text = trg_dir
            xml_trg_dir.set('updated','yes')
        xml_tree.write(xml_conf_file)  
        
    # Run sen2cor for one file
    def sen2cor(self, src_dir,res=20,trg_dir='DEFAULT'):
        '''
        src_dir: Path to the Sentinel-2 data folder (Attention if main folder was renamed: *_1C_* as part of directory name is mandatory!).
        res:     Processing Resolution, either 10, 20 or 60 [m], by default: 20 [m].
        trg_dir: Path where the results should be stored, by default it is the same folder as the src.
        '''
    
        res = str(res)                                              # Transformation to String
        name = 'sen2cor'                                            # Set Parameter name (tool=sen2cor)
        version = '2.0.6'                                           # Set Parameter version (sen2cor)
        
        system = platform.system()                                  # Get OS system
        
        if system == 'Linux':
            try:
                cfghome = os.environ['XDG_CONFIG_HOME']             # Get Position of sen2cor-Configuration-File
            except: 
                cfghome = os.environ['HOME']                        # Get Position of sen2cor-Configuration-File
        elif system == 'Windows':
            cfghome = os.path.join(os.environ['USERPROFILE'],'documents')# Get Position of sen2cor-Configuration-File  
        else:
            print("ERROR: Not supported OS!")
            sys.exit(1)

        SEN2COR_HOME  = os.path.join(cfghome, 'sen2cor')            # Home Directory of sen2cor
        xml_conf_file =  os.path.join(SEN2COR_HOME,'cfg','L2A_GIPP.xml')# Load configuration file of sen2cor
        
        if trg_dir == 'DEFAULT' or os.path.exists(trg_dir):
            self.updateConfigFile(xml_conf_file,trg_dir)            # Update configuration file.
        else:
            print "Target Directory does not exist!"
            sys.exit(1)
                
        if system == 'Linux':           
            cmd = 'L2A_Process --resolution '+res+' '+src_dir       # Parameters for the command prompt (window)
            os.system(cmd)                                          # Run parameters in the command prompt (window) 
        elif system == 'Windows':
            cmd = 'L2A_Process --resolution '+res+' '+src_dir       # Parameters for the command prompt (window)
            os.system(cmd)                                          # Run parameters in the command prompt (window)  
    
